// toShare attendance.js
'use strict';
// uses rest, user

class attendance{
	constructor(){
		this.divInoutEditLayer =  document.querySelector('div.inout-edit-layer');
		this.buttonClose = this.divInoutEditLayer.querySelector('button.close');
		this.labelUserName = this.divInoutEditLayer.querySelector('h2');
		this.inputRadioClockIn = this.divInoutEditLayer.querySelector('input.clock-in');
		this.inputRadioClockOut = this.divInoutEditLayer.querySelector('input.clock-out');
		this.buttonClockIn = this.divInoutEditLayer.querySelector('button.clock-in');
		this.buttonClockOut = this.divInoutEditLayer.querySelector('button.clock-out');
		this.inputWhereabouts = this.divInoutEditLayer.querySelector('input.whereabouts');
		this.selectWhereabouts = this.divInoutEditLayer.querySelector('select.whereabouts-select');
		this.inputSchedule = this.divInoutEditLayer.querySelector('input.schedule');
		this.selectSchedule = this.divInoutEditLayer.querySelector('select.schedule-select');
		this.buttonClear = this.divInoutEditLayer.querySelector('button.clear');
		this.buttonFix = this.divInoutEditLayer.querySelector('button.fix');

		this.onOutsideClick = this.onOutsideClick.bind(this);
		this.onCloseClick = this.onCloseClick.bind(this);
		this.onClockInClick = this.onClockInClick.bind(this);
		this.onClockOutClick = this.onClockOutClick.bind(this);
		this.onWhereaboutsKeyup = this.onWhereaboutsKeyup.bind(this);
		this.onWhereaboutsSelect = this.onWhereaboutsSelect.bind(this);
		this.onScheduleKeyup = this.onScheduleKeyup.bind(this);
		this.onScheduleSelect = this.onScheduleSelect.bind(this);
		this.onClearClick = this.onClearClick.bind(this);
		this.onFixClick = this.onFixClick.bind(this);

		this.divInoutEditLayer.addEventListener('mousedown', this.onOutsideClick);
		this.divInoutEditLayer.addEventListener('mouseup', this.onOutsideClick);
		this.buttonClose.addEventListener('click', this.onCloseClick);
		this.buttonClockIn.addEventListener('click', this.onClockInClick);
		this.buttonClockOut.addEventListener('click', this.onClockOutClick);
		this.selectWhereabouts.addEventListener('keyup', this.onWhereaboutsKeyup);
		this.selectWhereabouts.addEventListener('change', this.onWhereaboutsSelect);
		this.selectSchedule.addEventListener('keyup', this.onScheduleKeyup);
		this.selectSchedule.addEventListener('change', this.onScheduleSelect);
		this.buttonClear.addEventListener('click', this.onClearClick);
		this.buttonFix.addEventListener('click', this.onFixClick);

		this.memoMouseOutside = false;
	}

	async showInoutEditLayer(userId){
		try{
			const atsUser = atsAttendance.findField('user_id', userId);
			this.labelUserName.innerText = atsUser.user_name;
			this.labelUserName.dataset.userId = userId;
			switch(atsUser.clock_status){
			case 0:
				this.labelUserName.dataset.attendance = 'clock-out';
				this.labelUserName.classList.remove('clock-in');
				this.labelUserName.classList.add('clock-out');
				break;
			case 1:
				this.labelUserName.dataset.attendance = 'clock-in';
				this.labelUserName.classList.remove('clock-out');
				this.labelUserName.classList.add('clock-in');
				break;
			}
			this.inputWhereabouts.value = atsUser.whereabouts;
			this.inputSchedule.value = atsUser.description;

			this.divInoutEditLayer.classList.remove('hidden');
			this.divInoutEditLayer.classList.add('visible');
			this.setWhereabouts();
			this.inputWhereabouts.select();
			this.inputWhereabouts.focus();
		}catch(e){confirm(e.message);}
	}
	onOutsideClick(event){
		if(event.currentTarget===event.target){
			if(this.memoMouseOutside){
				this.onCloseClick();
			}else{
		 		this.memoMouseOutside = true;
			}
		}else{
			this.memoMouseOutside = false;
		}
	}
	onCloseClick(){
		this.divInoutEditLayer.classList.remove('visible');
		this.divInoutEditLayer.classList.add('hidden');
	}
	onClockInClick(event){
		this.labelUserName.dataset.attendance = 'clock-in';
		this.labelUserName.classList.remove('clock-out');
//		this.labelUserName.classList.add('clock-in');
		this.namehighlight(this.labelUserName, 'clock-in');
	}
	onClockOutClick(event){
		this.labelUserName.dataset.attendance = 'clock-out';
		this.labelUserName.classList.remove('clock-in');
//		this.labelUserName.classList.add('clock-out');
		this.namehighlight(this.labelUserName, 'clock-out');
	}
	async setWhereabouts(){
		if((atsWhereabouts.data.length==undefined)||isRegistryPgae)
		try{
			await atsWhereabouts.restGet();
			for(const atsWhereabout of atsWhereabouts.data){
				let userOption = document.createElement('option');
				userOption.innerText = atsWhereabout.whereabouts;
				this.selectWhereabouts.appendChild(userOption);
			};
		}catch(e){confirm(e.message);}
	}
	onWhereaboutsKeyup(event){
		switch(event.key){
			case 'Enter':{
				this.onWhereaboutsSelect();
				break;
			}
			case 'Escape':{
				this.inputWhereabouts.focus();
				break;
			}
		}
	}
	onWhereaboutsSelect(){
		if(event.currentTarget===event.target){
			this.inputWhereabouts.value = this.selectWhereabouts.value;
			this.inputWhereabouts.focus();
			this.selectWhereabouts.selectedIndex = 0;
		}
	}
	onScheduleKeyup(event){
		switch(event.key){
			case 'Enter':{
				this.onScheduleSelect();
				break;
			}
			case 'Escape':{
				this.inputSchedule.focus();
				break;
			}
		}
	}
	onScheduleSelect(event){
		if(event.currentTarget===event.target){
			this.inputSchedule.value = this.selectSchedule.value;
			this.inputSchedule.focus();
			this.selectSchedule.selectedIndex = 0;
		}
	}
	onClearClick(){
		this.inputWhereabouts.value = '';
		this.inputSchedule.value='';
	}
	async onFixClick(){
		try{
			await atsAttendance.restPut(
				Number(this.labelUserName.dataset.userId),
				((this.labelUserName.dataset.attendance == 'clock-in') ? 1: 0 ),
				this.inputWhereabouts.value,
				this.inputSchedule.value
			);
			if(isInoutPage){
				attendanceUpdateLoad();
			}else{
				await atsAttendance.restGet(atsCurrentUserId)
				updateUserAttendance();
			};
		}catch(e){confirm(e.message);}
		this.divInoutEditLayer.classList.remove('visible');
		this.divInoutEditLayer.classList.add('hidden');
	}
	namehighlight(element, setStyle){
		element.classList.add('namehighlightStart');
		setTimeout(()=>{
			element.classList.remove('namehighlightStart');
			element.classList.add(setStyle);
		}, 100);
	}
}

const isInoutPage = document.location.pathname == '/inoutboard/'
const isRegistryPgae = document.location.pathname == '/registry/whereabouts'
const inoutEdit = new attendance();
if(!isInoutPage)inoutEdit.showInoutEditLayer(atsCurrentUserId);
function showInoutEditLayer(userId){
	inoutEdit.showInoutEditLayer(userId)
}

